@@ -1,9 +1,9 @@
 package Net::DNS::SEC::DSA;
 
 #
-# $Id: DSA.pm 1669 2018-04-27 10:17:13Z willem $
+# $Id: DSA.pm 1672 2018-05-02 07:14:35Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1669 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1672 $)[1];
 
 
 =head1 NAME
@@ -70,9 +70,8 @@ sub sign {
 	my $evpkey = Net::DNS::SEC::libcrypto::EVP_PKEY_new();
 	Net::DNS::SEC::libcrypto::EVP_PKEY_assign_DSA( $evpkey, $dsa );
 
-	my $ASN1 = Net::DNS::SEC::libcrypto::EVP_sign( $sigdata, $evpkey, &$evpmd );
-	my $l = 2 + unpack 'x3 C', $ASN1;
-	pack 'C a* a*', $t, map _BERunwrap($_), unpack "x2 a$l a*", $ASN1;
+	my $asn1 = Net::DNS::SEC::libcrypto::EVP_sign( $sigdata, $evpkey, &$evpmd );
+	_ASN1decode( $asn1, $t );
 }
 
 
@@ -96,23 +95,32 @@ sub verify {
 	my $evpkey = Net::DNS::SEC::libcrypto::EVP_PKEY_new();
 	Net::DNS::SEC::libcrypto::EVP_PKEY_assign_DSA( $evpkey, $dsa );
 
-	my ( $r, $s ) = map _BERwrap($_), unpack 'x a20 a20', $sigbin;
-	my $ASN1 = pack 'n a* a*', 0x3000 + length($r) + length($s), $r, $s;
-
-	Net::DNS::SEC::libcrypto::EVP_verify( $sigdata, $ASN1, $evpkey, &$evpmd );
+	my $asn1 = _ASN1encode($sigbin);
+	Net::DNS::SEC::libcrypto::EVP_verify( $sigdata, $asn1, $evpkey, &$evpmd );
 }
 
 
-sub _BERwrap {
-	my $s = shift;
-	$s =~ s/^\000+(?=.)//;
-	$s =~ s/^(?=[\200-\377])/\000/;
-	pack 'C2 a*', 2, length($s), $s;
+########################################
+
+sub _ASN1encode {
+	my @part = unpack 'x a20 a20', shift;			# discard "t"
+	my $length;
+	foreach (@part) {
+		s/^[\000]+//;
+		s/^$/\000/;
+		s/^(?=[\200-\377])/\000/;
+		$_ = pack 'C2 a*', 2, length, $_;
+		$length += length;
+	}
+	pack 'C2 a* a*', 0x30, $length, @part;
 }
 
-sub _BERunwrap {
-	my ( $l, $s ) = unpack 'x C a*', shift;
-	substr pack( "x20 a$l", $s ), -20;
+sub _ASN1decode {
+	my ( $asn1, $t ) = @_;
+	my $n	 = unpack 'x3 C',	   $asn1;
+	my $m	 = unpack "x5 x$n C",	   $asn1;
+	my @part = unpack "x4 a$n x2 a$m", $asn1;
+	pack 'C a* a*', $t, map substr( pack( 'x20 a*', $_ ), -20 ), @part;
 }
 
 
